﻿using System;
using System.IO;

using Newtonsoft.Json;

namespace TestProject
{
    /// <summary>
    /// 메모리 스트림 JSON 컨버터
    /// </summary>
    public class MemoryStreamJSONConverter : JsonConverter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 변환 가능 여부 구하기 - CanConvert(objectType)

        /// <summary>
        /// 변환 가능 여부 구하기
        /// </summary>
        /// <param name="objectType">객체 타입</param>
        /// <returns>변환 가능 여부</returns>
        public override bool CanConvert(Type objectType)
        {
            return typeof(MemoryStream).IsAssignableFrom(objectType);
        }

        #endregion
        #region JSON 읽기 - ReadJson(jsonReader, objectType, existingValue, jsonSerializer)

        /// <summary>
        /// JSON 읽기
        /// </summary>
        /// <param name="jsonReader">JSON 리더</param>
        /// <param name="objectType">객체 타입</param>
        /// <param name="existingValue">기존 값</param>
        /// <param name="jsonSerializer">JSON 직렬화기</param>
        /// <returns>객체</returns>
        public override object ReadJson(JsonReader jsonReader, Type objectType, object existingValue, JsonSerializer jsonSerializer)
        {
            byte[] byteArray = jsonSerializer.Deserialize<byte[]>(jsonReader);

            return byteArray != null ? new MemoryStream(byteArray) : new MemoryStream();
        }

        #endregion
        #region JSON 쓰기 - WriteJson(jsonWriter, value, jsonSerializer)

        /// <summary>
        /// JSON 쓰기
        /// </summary>
        /// <param name="jsonWriter">JSON 라이터</param>
        /// <param name="value">값</param>
        /// <param name="jsonSerializer">JSON 직렬화기</param>
        public override void WriteJson(JsonWriter jsonWriter, object value, JsonSerializer jsonSerializer)
        {
            byte[] byteArray = ((MemoryStream)value).ToArray();

            jsonSerializer.Serialize(jsonWriter, byteArray);
        }

        #endregion
    }
}